@@ -26,6 +26,7 @@ from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import fileutils
+from oslo_utils import netutils
 
 from ironic.common import dhcp_factory
 from ironic.common import exception
@@ -350,11 +351,9 @@ def create_pxe_config(task, pxe_options, template=None, ipxe_enabled=False):
         pxe_config_root_tag = '(( ROOT ))'
         pxe_config_disk_ident = '(( DISK_IDENTIFIER ))'
 
-        # Determine the appropriate commands based on the CPU architecture
-        arch = task.node.properties.get('cpu_arch', 'x86_64')
         commands = {
-            'linux_cmd': 'linuxefi' if arch != 'aarch64' else 'linux',
-            'initrd_cmd': 'initrdefi' if arch != 'aarch64' else 'initrd'
+            'linux_cmd': 'linux',
+            'initrd_cmd': 'initrd'
         }
         pxe_options.update(commands)
     else:
@@ -1089,10 +1088,8 @@ def get_volume_pxe_options(task):
         return __return_item_or_first_if_list(properties.get(key + 's', ''))
 
     def __format_portal(portal, iqn, lun):
-        if ':' in portal:
-            host, port = portal.split(':')
-        else:
-            host = portal
+        host, port = netutils.parse_host_port(portal)
+        if port is None:
             port = ''
         return ("iscsi:%(host)s::%(port)s:%(lun)x:%(iqn)s" %
                 {'host': host, 'port': port, 'lun': lun, 'iqn': iqn})
@@ -1298,7 +1295,10 @@ class TFTPImageCache(image_cache.ImageCache):
             # MiB -> B
             cache_size=CONF.pxe.image_cache_size * 1024 * 1024,
             # min -> sec
-            cache_ttl=CONF.pxe.image_cache_ttl * 60)
+            cache_ttl=CONF.pxe.image_cache_ttl * 60,
+            # This is a cache for kernel, initramfs and anaconda artifacts,
+            # none of which can be validated
+            disable_validation=True)
 
 
 def cache_ramdisk_kernel(task, pxe_info, ipxe_enabled=False):
