@@ -25,6 +25,7 @@ from ironic_lib import metrics_utils
 from ironic_lib import utils as ironic_utils
 from oslo_log import log as logging
 from oslo_utils import importutils
+from oslo_utils import netutils
 
 from ironic.common import boot_devices
 from ironic.common import exception
@@ -41,6 +42,7 @@ from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules.irmc import common as irmc_common
 from ironic.drivers.modules.irmc import management as irmc_management
 from ironic.drivers.modules import pxe
+from ironic.drivers import utils as driver_utils
 
 
 scci = importutils.try_import('scciclient.irmc.scci')
@@ -57,14 +59,13 @@ LOG = logging.getLogger(__name__)
 METRICS = metrics_utils.get_metrics_logger(__name__)
 
 REQUIRED_PROPERTIES = {
-    'irmc_deploy_iso': _("Deployment ISO image file name. "
-                         "Required."),
+    'deploy_iso': _("Deployment ISO image file name. Required."),
 }
 
 RESCUE_PROPERTIES = {
-    'irmc_rescue_iso': _("UUID (from Glance) of the rescue ISO. Only "
-                         "required if rescue mode is being used and ironic "
-                         "is managing booting the rescue ramdisk.")
+    'rescue_iso': _("UUID (from Glance) of the rescue ISO. Only "
+                    "required if rescue mode is being used and ironic "
+                    "is managing booting the rescue ramdisk.")
 }
 
 OPTIONAL_PROPERTIES = {
@@ -80,9 +81,12 @@ OPTIONAL_PROPERTIES = {
           "the IPv4 subnet mask that the storage network is configured to "
           "utilize, in a range between 1 and 31 inclusive. This is necessary "
           "for booting a node from a remote iSCSI volume. Optional."),
+    'kernel_append_params': driver_utils.KERNEL_APPEND_PARAMS_DESCRIPTION %
+    {'option_group': 'irmc'},
 }
 
 COMMON_PROPERTIES = REQUIRED_PROPERTIES.copy()
+COMMON_PROPERTIES.update(driver_utils.OPTIONAL_PROPERTIES)
 COMMON_PROPERTIES.update(OPTIONAL_PROPERTIES)
 
 
@@ -137,15 +141,11 @@ def _parse_driver_info(node, mode='deploy'):
     :raises: InvalidParameterValue, if any of the parameters have invalid
         value.
     """
-    d_info = node.driver_info
     deploy_info = {}
 
-    if mode == 'deploy':
-        image_iso = d_info.get('irmc_deploy_iso')
-        deploy_info['irmc_deploy_iso'] = image_iso
-    else:
-        image_iso = d_info.get('irmc_rescue_iso')
-        deploy_info['irmc_rescue_iso'] = image_iso
+    image_iso = driver_utils.get_agent_iso(node, mode,
+                                           deprecated_prefix='irmc')
+    deploy_info[f'{mode}_iso'] = image_iso
 
     error_msg = (_("Error validating iRMC virtual media for %s. Some "
                    "parameters were missing in node's driver_info") % mode)
@@ -162,6 +162,15 @@ def _parse_driver_info(node, mode='deploy'):
                     'node': node.uuid})
             raise exception.InvalidParameterValue(msg)
 
+    kernel_params = driver_utils.get_kernel_append_params(
+        node, default=CONF.irmc.kernel_append_params)
+    if kernel_params is None:
+        LOG.warning('Relying on [pxe]kernel_append_params in the iRMC '
+                    'hardware type is deprecated, please set '
+                    '[irmc]kernel_append_params')
+        kernel_params = CONF.pxe.kernel_append_params
+    deploy_info['kernel_append_params'] = kernel_params
+
     return deploy_info
 
 
@@ -177,16 +186,17 @@ def _parse_instance_info(node):
     :raises: InvalidParameterValue, if any of the parameters have invalid
         value.
     """
-    i_info = node.instance_info
     deploy_info = {}
 
-    if i_info.get('irmc_boot_iso'):
-        deploy_info['irmc_boot_iso'] = i_info['irmc_boot_iso']
+    boot_iso = driver_utils.get_field(node, 'boot_iso',
+                                      deprecated_prefix='irmc',
+                                      collection='instance_info')
+    if boot_iso:
+        deploy_info['boot_iso'] = boot_iso
 
-        if _is_image_href_ordinary_file_name(
-                deploy_info['irmc_boot_iso']):
+        if _is_image_href_ordinary_file_name(boot_iso):
             boot_iso = os.path.join(CONF.irmc.remote_image_share_root,
-                                    deploy_info['irmc_boot_iso'])
+                                    boot_iso)
 
             if not os.path.isfile(boot_iso):
                 msg = (_("Boot ISO file, %(boot_iso)s, "
@@ -238,11 +248,7 @@ def _setup_vmedia(task, mode, ramdisk_options):
     :raises: InvalidParameterValue if the validation of the
         PowerInterface or ManagementInterface fails.
     """
-
-    if mode == 'rescue':
-        iso = task.node.driver_info['irmc_rescue_iso']
-    else:
-        iso = task.node.driver_info['irmc_deploy_iso']
+    iso = driver_utils.get_agent_iso(task.node, mode, deprecated_prefix='irmc')
 
     if _is_image_href_ordinary_file_name(iso):
         iso_file = iso
@@ -278,20 +284,20 @@ def _prepare_boot_iso(task, root_uuid):
        for BIOS boot_mode failed.
     """
     deploy_info = _parse_deploy_info(task.node)
-    driver_internal_info = task.node.driver_internal_info
 
     # fetch boot iso
-    if deploy_info.get('irmc_boot_iso'):
-        boot_iso_href = deploy_info['irmc_boot_iso']
+    if deploy_info.get('boot_iso'):
+        boot_iso_href = deploy_info['boot_iso']
         if _is_image_href_ordinary_file_name(boot_iso_href):
-            driver_internal_info['irmc_boot_iso'] = boot_iso_href
+            task.node.set_driver_internal_info('boot_iso', boot_iso_href)
         else:
             boot_iso_filename = _get_iso_name(task.node, label='boot')
             boot_iso_fullpathname = os.path.join(
                 CONF.irmc.remote_image_share_root, boot_iso_filename)
             images.fetch(task.context, boot_iso_href, boot_iso_fullpathname)
 
-            driver_internal_info['irmc_boot_iso'] = boot_iso_filename
+            task.node.set_driver_internal_info('boot_iso',
+                                               boot_iso_filename)
 
     # create boot iso
     else:
@@ -304,9 +310,9 @@ def _prepare_boot_iso(task, root_uuid):
         ramdisk_href = (task.node.instance_info.get('ramdisk')
                         or image_properties['ramdisk_id'])
 
-        deploy_iso_href = deploy_info['irmc_deploy_iso']
+        deploy_iso_href = deploy_info['deploy_iso']
         boot_mode = boot_mode_utils.get_boot_mode(task.node)
-        kernel_params = CONF.pxe.pxe_append_params
+        kernel_params = deploy_info['kernel_append_params']
 
         boot_iso_filename = _get_iso_name(task.node, label='boot')
         boot_iso_fullpathname = os.path.join(
@@ -319,10 +325,10 @@ def _prepare_boot_iso(task, root_uuid):
                                kernel_params=kernel_params,
                                boot_mode=boot_mode)
 
-        driver_internal_info['irmc_boot_iso'] = boot_iso_filename
+        task.node.set_driver_internal_info('boot_iso',
+                                           boot_iso_filename)
 
-    # save driver_internal_info['irmc_boot_iso']
-    task.node.driver_internal_info = driver_internal_info
+    # save driver_internal_info['boot_iso']
     task.node.save()
 
 
@@ -370,10 +376,9 @@ def attach_boot_iso_if_needed(task):
     """Attaches boot ISO for a deployed node if it exists.
 
     This method checks the instance info of the bare metal node for a
-    boot ISO. If the instance info has a value of key 'irmc_boot_iso',
-    it indicates that 'boot_option' is 'netboot'. Threfore it attaches
-    the boot ISO on the bare metal node and then sets the node to boot from
-    virtual media cdrom.
+    boot ISO. If the instance info has a value of key 'boot_iso',
+    it indicates ramdisk deploy. Therefore it attaches the boot ISO on the bare
+    metal node and then sets the node to boot from virtual media cdrom.
 
     :param task: a TaskManager instance containing the node to act on.
     :raises: IRMCOperationError if attaching virtual media failed.
@@ -383,8 +388,10 @@ def attach_boot_iso_if_needed(task):
     d_info = task.node.driver_internal_info
     node_state = task.node.provision_state
 
-    if 'irmc_boot_iso' in d_info and node_state == states.ACTIVE:
-        _setup_vmedia_for_boot(task, d_info['irmc_boot_iso'])
+    # Internal field, no deprecation
+    boot_iso = d_info.get('boot_iso') or d_info.get('irmc_boot_iso')
+    if boot_iso and node_state == states.ACTIVE:
+        _setup_vmedia_for_boot(task, boot_iso)
         manager_utils.node_set_boot_device(task, boot_devices.CDROM)
 
 
@@ -580,8 +587,6 @@ class IRMCVolumeBootMixIn(object):
     used by VIOM (Virtual I/O Manager) library of SCCI client.
     """
 
-    supported = False
-
     def _validate_volume_boot(self, task):
         """Validate information for volume boot with this interface.
 
@@ -839,12 +844,9 @@ class IRMCVolumeBootMixIn(object):
     def _set_iscsi_target(self, task, viom_conf, target):
         """Set information for iSCSI boot to VIOM configuration."""
         connectors = self._get_volume_connectors_by_type(task)
-        target_portal = target.properties['target_portal']
-        if ':' in target_portal:
-            target_host, target_port = target_portal.split(':')
-        else:
-            target_host = target_portal
-            target_port = None
+        target_host, target_port = netutils.parse_host_port(
+            target.properties['target_portal'])
+
         if target.properties.get('auth_method') == 'CHAP':
             chap_user = target.properties.get('auth_username')
             chap_secret = target.properties.get('auth_password')
@@ -900,8 +902,6 @@ class IRMCVolumeBootMixIn(object):
 class IRMCVirtualMediaBoot(base.BootInterface, IRMCVolumeBootMixIn):
     """iRMC Virtual Media boot-related actions."""
 
-    supported = False
-
     capabilities = ['iscsi_volume_boot', 'fibre_channel_volume_boot']
 
     def __init__(self):
@@ -943,14 +943,7 @@ class IRMCVirtualMediaBoot(base.BootInterface, IRMCVolumeBootMixIn):
             return
 
         d_info = _parse_deploy_info(task.node)
-        if task.node.driver_internal_info.get('is_whole_disk_image'):
-            props = []
-        elif service_utils.is_glance_image(d_info['image_source']):
-            props = ['kernel_id', 'ramdisk_id']
-        else:
-            props = ['kernel', 'ramdisk']
-        deploy_utils.validate_image_properties(task.context, d_info,
-                                               props)
+        deploy_utils.validate_image_properties(task, d_info)
 
     @METRICS.timer('IRMCVirtualMediaBoot.prepare_ramdisk')
     def prepare_ramdisk(self, task, ramdisk_params):
@@ -970,13 +963,7 @@ class IRMCVirtualMediaBoot(base.BootInterface, IRMCVolumeBootMixIn):
         :raises: IRMCOperationError, if some operation on iRMC fails.
         """
 
-        # NOTE(TheJulia): If this method is being called by something
-        # aside from deployment, clean and rescue, such as conductor takeover,
-        # we should treat this as a no-op and move on otherwise we would
-        # modify the state of the node due to virtual media operations.
-        if task.node.provision_state not in (states.DEPLOYING,
-                                             states.CLEANING,
-                                             states.RESCUING):
+        if not driver_utils.need_prepare_ramdisk(task.node):
             return
 
         # NOTE(tiendc): Before deploying, we need to backup BIOS config
@@ -998,7 +985,8 @@ class IRMCVirtualMediaBoot(base.BootInterface, IRMCVolumeBootMixIn):
         task.node.save()
 
         deploy_nic_mac = deploy_utils.get_single_nic_with_vif_port_id(task)
-        ramdisk_params['BOOTIF'] = deploy_nic_mac
+        if deploy_nic_mac is not None:
+            ramdisk_params['BOOTIF'] = deploy_nic_mac
 
         if task.node.provision_state == states.RESCUING:
             mode = 'rescue'
@@ -1045,13 +1033,12 @@ class IRMCVirtualMediaBoot(base.BootInterface, IRMCVolumeBootMixIn):
             manager_utils.node_set_boot_device(task, boot_devices.DISK,
                                                persistent=True)
         else:
-            driver_internal_info = node.driver_internal_info
-            root_uuid_or_disk_id = driver_internal_info['root_uuid_or_disk_id']
+            root_uuid_or_disk_id = node.driver_internal_info[
+                'root_uuid_or_disk_id']
             self._configure_vmedia_boot(task, root_uuid_or_disk_id)
 
         # Enable secure boot, if being requested
-        if deploy_utils.is_secure_boot_requested(node):
-            irmc_common.set_secure_boot_mode(node, enable=True)
+        boot_mode_utils.configure_secure_boot_if_needed(task)
 
     @METRICS.timer('IRMCVirtualMediaBoot.clean_up_instance')
     def clean_up_instance(self, task):
@@ -1069,14 +1056,12 @@ class IRMCVirtualMediaBoot(base.BootInterface, IRMCVolumeBootMixIn):
             return
 
         # Disable secure boot, if enabled secure boot
-        if deploy_utils.is_secure_boot_requested(task.node):
-            irmc_common.set_secure_boot_mode(task.node, enable=False)
+        boot_mode_utils.deconfigure_secure_boot_if_needed(task)
 
         _remove_share_file(_get_iso_name(task.node, label='boot'))
-        driver_internal_info = task.node.driver_internal_info
-        driver_internal_info.pop('irmc_boot_iso', None)
+        task.node.del_driver_internal_info('boot_iso')
+        task.node.del_driver_internal_info('irmc_boot_iso')
 
-        task.node.driver_internal_info = driver_internal_info
         task.node.save()
         _cleanup_vmedia_boot(task)
 
@@ -1085,7 +1070,7 @@ class IRMCVirtualMediaBoot(base.BootInterface, IRMCVolumeBootMixIn):
         node = task.node
         _prepare_boot_iso(task, root_uuid_or_disk_id)
         _setup_vmedia_for_boot(
-            task, node.driver_internal_info['irmc_boot_iso'])
+            task, node.driver_internal_info['boot_iso'])
         manager_utils.node_set_boot_device(task, boot_devices.CDROM,
                                            persistent=True)
 
@@ -1105,8 +1090,6 @@ class IRMCVirtualMediaBoot(base.BootInterface, IRMCVolumeBootMixIn):
 class IRMCPXEBoot(pxe.PXEBoot):
     """iRMC PXE boot."""
 
-    supported = False
-
     @METRICS.timer('IRMCPXEBoot.prepare_ramdisk')
     def prepare_ramdisk(self, task, ramdisk_params):
         """Prepares the boot of Ironic ramdisk using PXE.
@@ -1133,39 +1116,3 @@ class IRMCPXEBoot(pxe.PXEBoot):
             irmc_management.backup_bios_config(task)
 
         super(IRMCPXEBoot, self).prepare_ramdisk(task, ramdisk_params)
-
-    @METRICS.timer('IRMCPXEBoot.prepare_instance')
-    def prepare_instance(self, task):
-        """Prepares the boot of instance.
-
-        This method prepares the boot of the instance after reading
-        relevant information from the node's instance_info. In case of netboot,
-        it updates the dhcp entries and switches the PXE config. In case of
-        localboot, it cleans up the PXE config.
-
-        :param task: a task from TaskManager.
-        :returns: None
-        :raises: IRMCOperationError, if some operation on iRMC failed.
-        """
-
-        super(IRMCPXEBoot, self).prepare_instance(task)
-        node = task.node
-        if deploy_utils.is_secure_boot_requested(node):
-            irmc_common.set_secure_boot_mode(node, enable=True)
-
-    @METRICS.timer('IRMCPXEBoot.clean_up_instance')
-    def clean_up_instance(self, task):
-        """Cleans up the boot of instance.
-
-        This method cleans up the environment that was setup for booting
-        the instance. It unlinks the instance kernel/ramdisk in node's
-        directory in tftproot and removes the PXE config.
-
-        :param task: a task from TaskManager.
-        :raises: IRMCOperationError, if some operation on iRMC failed.
-        :returns: None
-        """
-        node = task.node
-        if deploy_utils.is_secure_boot_requested(node):
-            irmc_common.set_secure_boot_mode(node, enable=False)
-        super(IRMCPXEBoot, self).clean_up_instance(task)
